==> photobooth_app-2.0.2.tar.gz: photobooth_app-2.0.2/photobooth/services/printingservice.py <==
"""
Handle all media collection related functions
"""

import subprocess
import time

from .baseservice import BaseService
from .config import appconfig
from .mediacollection.mediaitem import MediaItem
from .mediacollectionservice import MediacollectionService
from .sseservice import SseEventFrontendNotification, SseService

TIMEOUT_PROCESS_RUN = 6  # command to print needs to complete within 6 seconds.


class PrintingService(BaseService):
    """Handle all image related stuff"""

    def __init__(self, sse_service: SseService, mediacollection_service: MediacollectionService):
        super().__init__(sse_service)

        # common objects
        self._mediacollection_service: MediacollectionService = mediacollection_service

        # custom service objects
        self._last_print_time = None
        self._printing_queue = None  # TODO: could add queue later

    def start(self):
        # unblock after restart immediately
        self._last_print_time = None

    def stop(self):
        pass

    def print(self, mediaitem: MediaItem):
        ## print mediaitem

        if not appconfig.hardwareinputoutput.printing_enabled:
            raise ConnectionRefusedError("Printing is disabled! Enable in config first.")

        # block queue new prints until configured time is over
        if self.is_blocked():
            raise BlockingIOError(f"Print request ignored! Wait {self.remaining_time_blocked():.0f}s before try again.")

        # filename absolute to print, use in printing command
        filename = mediaitem.path_full.absolute()

        try:
            # print command
            self._logger.info(f"printing {filename=}")

            completed_process = subprocess.run(
                str(appconfig.hardwareinputoutput.printing_command).format(filename=filename),
                capture_output=True,
                check=True,
                timeout=TIMEOUT_PROCESS_RUN,
                shell=True,  # needs to be shell so a string as command is accepted.
            )

            self._logger.info(f"cmd={completed_process.args}")
            self._logger.info(f"stdout={completed_process.stdout}")
            self._logger.debug(f"stderr={completed_process.stderr}")

            self._logger.info(f"print command started successfully {mediaitem}")

            # update last print time to calc block time on next run
            self._start_time_blocked()
        except Exception as exc:
            self._sse_service.dispatch_event(SseEventFrontendNotification(color="negative", message=f"{exc}", caption="Print Error"))
            raise RuntimeError(f"print failed, error {exc}") from exc

    def is_blocked(self):
        return self.remaining_time_blocked() > 0.0

    def remaining_time_blocked(self) -> float:
        if self._last_print_time is None:
            return 0.0

        delta = time.time() - self._last_print_time
        if delta >= appconfig.hardwareinputoutput.printing_blocked_time:
            # last print is longer than configured time in the past - return 0 to indicate no wait time
            return 0.0
        else:
            # there is some time to wait left.
            return appconfig.hardwareinputoutput.printing_blocked_time - delta

    def _start_time_blocked(self):
        self._last_print_time = time.time()

        # TODO: add some timer/coroutine to send regular update to UI with current remaining time blocked

    def _print_timer_fun(self):
        ## thread to send updates to client about remaining blocked time
        pass

==> photobooth_app-3.1.3.tar.gz: photobooth_app-3.1.3/photobooth/services/printingservice.py <==
"""
Handle all media collection related functions
"""

import subprocess
import time

from .baseservice import BaseService
from .config import appconfig
from .informationservice import InformationService
from .mediacollection.mediaitem import MediaItem
from .mediacollectionservice import MediacollectionService
from .sseservice import SseEventFrontendNotification, SseService

TIMEOUT_PROCESS_RUN = 6  # command to print needs to complete within 6 seconds.


class PrintingService(BaseService):
    """Handle all image related stuff"""

    def __init__(self, sse_service: SseService, mediacollection_service: MediacollectionService, information_service: InformationService):
        super().__init__(sse_service)

        # common objects
        self._mediacollection_service: MediacollectionService = mediacollection_service
        self._information_service: InformationService = information_service

        # custom service objects
        self._last_print_time = None
        self._last_printing_blocked_time = None
        self._printing_queue = None  # TODO: could add queue later

    def start(self):
        # unblock after restart immediately
        self._last_print_time = None

    def stop(self):
        pass

    def print(self, mediaitem: MediaItem, config_index: int = 0):
        """print mediaitem"""

        if not appconfig.hardwareinputoutput.printing_enabled:
            self._sse_service.dispatch_event(
                SseEventFrontendNotification(
                    color="negative",
                    message="Printing is disabled! Enable in config first.",
                    caption="Print Error",
                )
            )
            raise ConnectionRefusedError("Printing is disabled! Enable in config first.")

        # block queue new prints until configured time is over
        if self.is_blocked():
            self._sse_service.dispatch_event(
                SseEventFrontendNotification(
                    color="info",
                    message=f"Print request ignored! Wait {self.remaining_time_blocked():.0f}s before try again.",
                    caption="Print Error",
                )
            )
            raise BlockingIOError(f"Print request ignored! Wait {self.remaining_time_blocked():.0f}s before try again.")

        # get config
        try:
            config = appconfig.printer.print[config_index].processing
        except Exception as exc:
            self._logger.critical(f"could not find action configuration with index {config_index}, error {exc}")
            raise exc

        # filename absolute to print, use in printing command
        filename = mediaitem.path_full.absolute()

        try:
            # print command
            self._logger.info(f"printing {filename=}")

            completed_process = subprocess.run(
                str(config.printing_command).format(filename=filename),
                capture_output=True,
                check=True,
                timeout=TIMEOUT_PROCESS_RUN,
                shell=True,  # needs to be shell so a string as command is accepted.
            )

            self._logger.info(f"cmd={completed_process.args}")
            self._logger.info(f"stdout={completed_process.stdout}")
            self._logger.debug(f"stderr={completed_process.stderr}")

            self._logger.info(f"print command started successfully {mediaitem}")

            # update last print time to calc block time on next run
            self._start_time_blocked(config.printing_blocked_time)

            self._sse_service.dispatch_event(SseEventFrontendNotification(color="positive", message="Started printing...", spinner=True))
        except Exception as exc:
            self._sse_service.dispatch_event(SseEventFrontendNotification(color="negative", message=f"{exc}", caption="Print Error"))
            raise RuntimeError(f"print failed, error {exc}") from exc

        self._information_service.stats_counter_increment("prints")

    def is_blocked(self):
        return self.remaining_time_blocked() > 0.0

    def remaining_time_blocked(self) -> float:
        if self._last_print_time is None:
            return 0.0

        delta = time.time() - self._last_print_time
        if delta >= self._last_printing_blocked_time:
            # last print is longer than configured time in the past - return 0 to indicate no wait time
            return 0.0
        else:
            # there is some time to wait left.
            return self._last_printing_blocked_time - delta

    def _start_time_blocked(self, printing_blocked_time: int):
        self._last_print_time = time.time()
        self._last_printing_blocked_time = printing_blocked_time

        # TODO: add some timer/coroutine to send regular update to UI with current remaining time blocked

    def _print_timer_fun(self):
        ## thread to send updates to client about remaining blocked time
        pass
